/************************************************************************/
/* chara_messages.h							*/
/*                                                                      */
/* Defines all messages used in the CHARA control system along with the */
/* necessary data structures.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar  		                        */
/* Date   : Feb 2003							*/
/************************************************************************/

/*
 * NOTE.
 * 
 * All of these definitions and declarations where originally made
 * using 32 bit intel processors. Thus all the sizes are the same.
 * In early 2006 we installed the first 64 bit processor and this
 * has caused some problems. Here's the sizeof comparison:
 *
 *	Type		32 BIT		64 BIT
 * 	--------------------------------------
 * 	char 		1		1
 *	short int 	2		2
 *	long int 	4		8		***
 *	long long int 	8		8
 *	int 		4		4
 *	float 		4		4
 *	double 		8		8
 *	long double 	12		16		***
 *	Pointer 	4		8		***
 * 
 * Thus it is dangerous to use long int, long double, or pointers in messages
 * because they will not be the same length. This file was modifed accordingly
 * on Jan 04 2006.
 *
 * We really should be using uint16_t like types, but for the time being this
 * will take too long to make all the required changes. For now, we remove
 * all references to long int and long double.
 * 
 * Another concern is that on the 64 bit machine floats and doubles must lie 
 * on a 64 bit boundary. This means some of the structures may be different
 * sizes. Therefore, all structures have been modified so that all doubles
 * come first, followed by floats, then integers then chars and so on.
 * The test.c program has also been changed so that it prints out the size of
 * all structures.
 */

#ifndef __CHARA_MESSAGES__
#define __CHARA_MESSAGES__

#ifndef NUM_SCOPES
#define NUM_SCOPES 6
#endif 

/************************************************************************/
/* STANDARD messages 0-99						*/
/************************************************************************/

#define FIRST_STANDARD_MESSAGE	0

/* 
 * The first and simplest message is a no-operation message.
 * It does nothing and has no data.
 */

#define NOOP	(FIRST_STANDARD_MESSAGE+0)

/*
 * The next message is a MESSAGE. In the server case,
 * the data will be a string which will be sent to the text output
 * stream. On the client side it will display the string somehow.
 * The data will be the string, not including the terminating NULL.
 * Note that this is not really required as it is often the same
 * as another define in cliserv.h or charaui.h and it had better
 * be the same too!
 */

#ifndef MESSAGE
#define MESSAGE (FIRST_STANDARD_MESSAGE+1) 
#endif

/*
 * A ping message has no agument and asks for a return message of the
 * type MESSAGE saying we are alive.
 */

#define PING	(FIRST_STANDARD_MESSAGE+2)

/*
 * An instruction to send a NOOP command to all connected machines.
 */

#define ALL_NOOP (FIRST_STANDARD_MESSAGE+3)

/*
 * An instruction to PING all connected machines.
 */

#define ALL_PING (FIRST_STANDARD_MESSAGE+4)

/* 
 * Ask for or send status information. In the UI casse it sends the 
 * structure below which will be a copy of the text in the status window.
 * Other applications may have a different idea about what to send.
 * These status window numbers should not be here.
 */

#define UI_STATUS   (FIRST_STANDARD_MESSAGE+5)
#ifndef STATUS_LINES
#define STATUS_LINES	9
#define STATUS_COLS	79
#endif

struct s_status {
		  char line[STATUS_LINES][STATUS_COLS+1];
		};

/*
 * Set a signal flag. If going from Client to server sets the flag, if going 
 * from server to client can set flags within the client, or do nothing,
 * depending on the client. Meant for sending signals, like STOP NOW and
 * so on.
 * Data is an unsigned char, the number of the signal from 0 to 255.
 */

#define UI_SIGNAL   (FIRST_STANDARD_MESSAGE+6)

/*
 * This message is sent to all clients when a server either dies or
 * when it is simply told to stop running.
 */

#define SERVER_STOPPING (FIRST_STANDARD_MESSAGE+7)

/*
 * This is a message that requests that the server send another message
 * to all clients. The message to be sent is the data and is of type 
 * struct smessage. This message can have no data.
 */

#define SEND_MESSAGE_ALL_CLIENTS (FIRST_STANDARD_MESSAGE+8)

/* Go into or out of remote mode, no data */

#define REMOTE_MODE_ON (FIRST_STANDARD_MESSAGE+9)
#define REMOTE_MODE_OFF (FIRST_STANDARD_MESSAGE+10)

#define LAST_STANDARD_MESSAGE REMOTE_MODE_OFF

/************************************************************************/
/* CLOCK messages 100-199						*/
/************************************************************************/

#define FIRST_CLOCK_MESSAGE	100

/* Client -> Server  Send FLush command, no data  */

#define	FLUSH_FIFOS		(FIRST_CLOCK_MESSAGE+0)

/* Client->Server run clear_lost_counts() function, no data */

#define	MESS_CLEAR_LOST_COUNTS	(FIRST_CLOCK_MESSAGE+1)

/* 
 * Client->Server run in_set(). Sends two bytes as argument.
 * First byte is the port number, second byte is in_or_out.
 *
 * Server->Client result of in_set(), Argument is used to return a TRUE
 * or FALSE result in a single byte message.
 */

#define	IN_SET			(FIRST_CLOCK_MESSAGE+2)

/*
 * Client->Server run get_rt_time and send back result, an rt_time structure.
 * 
 * Server->Client result of get_rt_time;
 */

#define RT_TIME			(FIRST_CLOCK_MESSAGE+3)

/*
 * Client->Server run get_time_next_second
 * 
 * Server->Client sends back the time of next second as CHARA_TIME
 */

#define TIME_NEXT_SECOND	(FIRST_CLOCK_MESSAGE+4)

/*
 * Client->Server run set_rt_time to set RT clock, data is CHARA_TIME
 */

#define SET_RT_TIME		(FIRST_CLOCK_MESSAGE+5)

/*
 * Client->Server runs sync_times()
 */

#define SYNC_TIMES		(FIRST_CLOCK_MESSAGE+6)

/*
 * CLient->Server runs call_sync_handler()
 */

#define CALL_SYNC_HANDLER	(FIRST_CLOCK_MESSAGE+7)

/* 
 * Client->Server calls set_lpt(), data single byte.
 * Server->Client resposne from server with LPT value, data single byte.
 */

#define MESS_SET_LPT		(FIRST_CLOCK_MESSAGE+8)

/*
 * Client -> Server calls get_lpt() Server then uses MESS_SET_LPT message
 * to return result.
 */

#define MESS_GET_LPT		(FIRST_CLOCK_MESSAGE+9)

/*
 * Client->Server calls set_bit_lpt() Server then uses SET_LPT message
 * to return the resulting LPT value.
 */

#define SET_LPT_BIT		(FIRST_CLOCK_MESSAGE+10)

/*
 * Client->Server calls set_bit_lpt() Server then uses SET_LPT message
 * to return the resulting LPT value.
 */

#define UNSET_LPT_BIT		(FIRST_CLOCK_MESSAGE+11)

/* Client->Server request reinitialization of the clock. */

#define REINIT_CLOCK		(FIRST_CLOCK_MESSAGE+12)

/* Last clock message */

#define LAST_CLOCK_MESSAGE	REINIT_CLOCK

/************************************************************************/
/* GPS based messages 200-299						*/
/************************************************************************/

#define FIRST_GPS_MESSAGE	200

/* All Client->Server and calls routines with no arguments */

#define GPS_START		(FIRST_GPS_MESSAGE+0)
#define GPS_STOP		(FIRST_GPS_MESSAGE+1)
#define GPS_SID			(FIRST_GPS_MESSAGE+2)
#define GPS_SYNC		(FIRST_GPS_MESSAGE+3)

/* 
 * Client->Server requesting a sync operation on a particular process 
 *  Data is the name of the process
 */

#define GPS_MSYNC		(FIRST_GPS_MESSAGE+4)

#define LAST_GPS_MESSAGE	GPS_MSYNC

/************************************************************************/
/* IRIMAGE messages 300-399						*/
/************************************************************************/

#define FIRST_IRIMAGE_MESSAGE 300

/* Client->Server Set bandpass frequency */

struct s_bp_filter {
			float freq;
			float width;
			unsigned char channel; /* H_BAND or K_BAND */
		   };

#define IRIMAGE_SET_BP_FILTER		(FIRST_IRIMAGE_MESSAGE+0)

/* Client->Server Set low pass frequency */

struct s_lp_filter {
			float freq;
			unsigned char channel; /* H_BAND or K_BAND */
		   };
			
#define IRIMAGE_SET_LP_FILTER		(FIRST_IRIMAGE_MESSAGE+1)

/* client->Server Set NIRO sample time in microseconds , data is an integer */

#define IRIMAGE_SET_NIRO_SAMPLE		(FIRST_IRIMAGE_MESSAGE+2)

/* Client->Server Set NIRO wait time in microseconds , data is an integer */

#define IRIMAGE_SET_NIRO_WAIT		(FIRST_IRIMAGE_MESSAGE+3)

/* 
 * Client->Server toggle dither ON/OFF no data 
 */

#define IRIMAGE_TOGGLE_DITHER		(FIRST_IRIMAGE_MESSAGE+4)

/* Client->Server Set new scan dither, data as per structue below */

struct s_scan_dither {
			float	lambda;	/* um */
			float	freq;	/* Hz */
			float	f6db;	/* Hz */
		    };

#define IRIMAGE_SCAN_DITHER		(FIRST_IRIMAGE_MESSAGE+5)

/* Clilent->Server Set NIRO mode, argument an integer, one of */

#define NIRO_NORM               0
#define NIRO_SCAN               1
#define NIRO_ASYN               2
#define NIRO_PICT               3
#define NIRO_SYNC               4

#define IRIMAGE_NIRO_MODE		(FIRST_IRIMAGE_MESSAGE+6)

/*
 * Client->Server, request current NIRO mode, no data
 * Server->Client, result of NIRO mode request, data integer
 */

#define IRIMAGE_GET_NIRO_MODE		(FIRST_IRIMAGE_MESSAGE+7)

/* Client->Server, request NIRO resync */

#define IRIMAGE_SYNC_NIRO		(FIRST_IRIMAGE_MESSAGE+8)

/* 
 * Client->Server, request data collection.
 * Data type depends on current NIRO mode.
 * For PICT mode the data is
 */

struct s_data_pict {
			float memory; /* 0.0 to 1.0 */
			int display_mode; /* LIN, LOG, EXP */
			char	filename[256]; /* Zero length for none */
			char	display[256];  /* Must be something */
		   };

/* For NORM mode the data is */

struct s_data_norm {
			float	minsnr; /* Default 0.0 */
			int	width;	/* Default 50 */
			char	skip_save; /* Default FALSE */
			char	skip_display; /* Default FALSE */
			char	display_non_interp; /* Default FALSE */
			char	skip_low_weight; /* Save only good scans? */
			char	use_target_memory; /* Guess? */
			char	display[256];  /* Must be something */
			char	beam5[256];	/* Which scope is beam 3? */
			char	beam6[256];	/* Which scope is beam 4? */
			char	obj_type[80];	/* Type of object */
		   };
			
#define IRIMAGE_COLLECT_DATA		(FIRST_IRIMAGE_MESSAGE+9)

/* 
 * Server->Client for sending status on PICT mode display data is structure:
 */

struct s_pict_status_internal {
			float	x;
			float	y;
			float	amp;
			float	x0;
			float	y0;
			float	fwhm;
			int	min;
			int	max;
			int	in;
			int	tot;
			      };

struct s_pict_status{
			struct s_pict_status_internal ir1;
			struct s_pict_status_internal ir2;
			float	memory;
			int	sample;
			char	show;
		    };

#define IRIMAGE_PICT_STATUS	     	(FIRST_IRIMAGE_MESSAGE+10)

/* Server->client message to stop displaying pict status */

#define IRIMAGE_PICT_STATUS_STOP     	(FIRST_IRIMAGE_MESSAGE+11)
			
/* Server-> CLIent for sending status data on NORM mode display data is */

struct	s_norm_status {
			double	position;
			float	snr_now;
			float	snr_max;
			float	snr_time;
			float	vup;
			float	vdn;
			float   pos;
			float   weight;
			float   min_weight;
			float   max_error;
			float   gain;
			float	fringetarg;
			int	scans;
			int	offset;
			char	filename[80];
			char	hold;
			char	comment[80];
			char	dither_is_on;
			char    track_servo_on;
		      };

#define IRIMAGE_NORM_STATUS		(FIRST_IRIMAGE_MESSAGE+12)
			
/* Server->client message to stop displaying norm status */

#define IRIMAGE_NORM_STATUS_STOP     	(FIRST_IRIMAGE_MESSAGE+13)
			
/* 
 * Client->Server open tracking socket argument used to be structure below 
 * but is now no data.
 */

struct s_socket_name {
			int	socket;
			char	machine[256];
		     };

#define IRIMAGE_OPEN_TRACKING_SOCKET     (FIRST_IRIMAGE_MESSAGE+14)

/* CLient->Server for controlling the display socket, data s_socket_name */

#define IRIMAGE_OPEN_DISPLAY_SOCKET      (FIRST_IRIMAGE_MESSAGE+15)

/* CLient->Server for requesting which pixels we are using on NIRO */

#define IRIMAGE_GET_NIRO_PIXELS      	(FIRST_IRIMAGE_MESSAGE+16)

/* Either direction for controling setting NIRO pixels we use */

struct s_niro_pixels {
			int size;
			int row;
			int col1;
			int col2;
		     };

#define IRIMAGE_SET_NIRO_PIXELS		(FIRST_IRIMAGE_MESSAGE+17)

/* 
 * Client->Server request DITHER_STATE message, no data 
 * Server->Client return dither state as single byte.
 */

#define IRIMAGE_DITHER_STATE		(FIRST_IRIMAGE_MESSAGE+18)

/* 
 * Client->Server, request NIRO sample time, no data.
 * Server should respond with a IRIMAGE_SET_NIRO_SAMPLE message.
 */

#define IRIMAGE_GET_NIRO_SAMPLE		(FIRST_IRIMAGE_MESSAGE+19)

/* 
 * Client->Server, request NIRO wait time, no data.
 * Server should respond with a IRIMAGE_SET_NIRO_NREADS message.
 */

#define IRIMAGE_GET_NIRO_WAIT		(FIRST_IRIMAGE_MESSAGE+20)

/* Client->Serevr, set beam combiner, data int */

#ifndef CHARA_CLASSIC
#define CHARA_CLASSIC           0
#define FLUOR                   1
#endif

#define IRIMAGE_SET_BEAM_COMBINER	(FIRST_IRIMAGE_MESSAGE+21)

/* Client->Server for controlling what kind of PS we see */

#define IRIMAGE_PS_UNFITLERED		(FIRST_IRIMAGE_MESSAGE+22)

#define IRIMAGE_PS_FITLERED		(FIRST_IRIMAGE_MESSAGE+23)

/* Client->Server Set new linear dither, data as per structue below */

struct s_linear_dither {
			int	length;	/* steps */
			int	inc;	/* incriment */
		    };

#define IRIMAGE_LINEAR_DITHER		(FIRST_IRIMAGE_MESSAGE+24)

/* Client->Server Set new phase dither, data integer number of steps */

#define IRIMAGE_PHASE_DITHER		(FIRST_IRIMAGE_MESSAGE+25)

#define IRIMAGE_RESET_FILTER		(FIRST_IRIMAGE_MESSAGE+26)

/* CLient->Server for controlling the niro socket, data s_socket_name */

#define IRIMAGE_OPEN_NIRO_SOCKET      (FIRST_IRIMAGE_MESSAGE+27)

/* Client->Server for asking for power of two for number of reads, no data */

#define IRIMAGE_GET_NIRO_NREADS	      (FIRST_IRIMAGE_MESSAGE+28)

/*
 * Client->Server for setting number of reads, data struct
 * Server->Client response to IRIMAGE_GET_NIRO_NREADS, data struct
 */
 
struct s_niro_nreads {
			int	nreads;
			int	nloops;
			int	nskips;
			int	nboundary;
		    };

#define IRIMAGE_SET_NIRO_NREADS       (FIRST_IRIMAGE_MESSAGE+29)

/* Client->Server request mean count calculation. Data integer samples */

#define IRIMAGE_GET_MEAN_COUNTS		(FIRST_IRIMAGE_MESSAGE+30)

/* Client->Server for setting a standard NIRO type. Data integer, one of */

#define NIRO_UNKNOWN		-1
#define NIRO_1000HzNPIX2	0
#define NIRO_750HzNPIX2		1
#define NIRO_500HzNPIX2		2
#define NIRO_250HzNPIX2		3
#define NIRO_1000HzNPIX1	4
#define NIRO_750HzNPIX1		5
#define NIRO_500HzNPIX1		6
#define NIRO_250HzNPIX1		7
#define NUM_NIRO_STANDARD_TYPES	8

#define IRIMAGE_SET_NIRO_STANDARD_TYPE	(FIRST_IRIMAGE_MESSAGE+31)

/* Client->Server for changing the tracking mode Data integer and 
 * one of the following
 */

#ifndef FRINGE_SCAN
#define FRINGE_SCAN              	0 /* Use long scans */
#define FRINGE_PHASE             	1 /* Tracking to phase */
#endif

#define IRIMAGE_SET_FRINGE_MODE		(FIRST_IRIMAGE_MESSAGE+32)

#define IRIMAGE_ARM_NIRO		(FIRST_IRIMAGE_MESSAGE+33)

#define IRIMAGE_SHOW_PS			(FIRST_IRIMAGE_MESSAGE+34)
#define IRIMAGE_SHOW_ENV		(FIRST_IRIMAGE_MESSAGE+35)

/* Client->Server for changing the number of data scans, data struct */

struct s_set_data_scans {
	int data_scans;
	int shutter_scans_a;
	int shutter_scans_b;
		        };

#define IRIMAGE_SET_DATA_SCANS		(FIRST_IRIMAGE_MESSAGE+36)
                                                                                
/* 
 * Client->Server Setting fast dither on or off, data unsigned char.
 * Server->Client MEssage to inform client of fast dither state, unsigned char
 */

#define IRIMAGE_SET_FAST_DITHER		(FIRST_IRIMAGE_MESSAGE+37)

/* Client->Server, request server to send IRIMAGE_DITHER_STATE. */

#define IRIMAGE_FAST_DITHER_STATE	(FIRST_IRIMAGE_MESSAGE+38)

/* Client->Server set tracking cart, data an int. */

#define IRIMAGE_SET_TRACKING_CART	(FIRST_IRIMAGE_MESSAGE+39)

/* Client-> server set servo direction. data an int, only sign used. */

#define IRIMAGE_SET_DIRECTION		(FIRST_IRIMAGE_MESSAGE+40)

/* Client->server ask for current niro reset delay, no data */

#define IRIMAGE_GET_NIRO_RESET_DELAY	(FIRST_IRIMAGE_MESSAGE+41)

/* 
 * Server-> Client reply to previous, data integer
 * Client->Server set niro reset delay, data integer
 */

#define IRIMAGE_SET_NIRO_RESET_DELAY	(FIRST_IRIMAGE_MESSAGE+42)

/* Client->server ask for current niro reset time, no data */

#define IRIMAGE_GET_NIRO_RESET_TIME	(FIRST_IRIMAGE_MESSAGE+43)

/* 
 * Server-> Client reply to previous, data integer
 * Client->Server set niro reset time, data integer
 */

#define IRIMAGE_SET_NIRO_RESET_TIME	(FIRST_IRIMAGE_MESSAGE+44)

#define LAST_IRIMAGE_MESSAGE		IRIMAGE_SET_NIRO_RESET_TIME

/************************************************************************/
/* And now we define the signals that IRIMAGE knows about.		*/
/************************************************************************/

/* These signals are used in the PICT display routine */

#define IRIMAGE_SIGNAL_MAKE_BIAS	1
#define IRIMAGE_SIGNAL_ZERO_BIAS	2
#define IRIMAGE_SIGNAL_SHOW_PICTURE	3
#define IRIMAGE_SIGNAL_AUTOSCALE	4
#define IRIMAGE_SIGNAL_SHOW_DIFF	5
#define IRIMAGE_SIGNAL_SHOW_FIT		6

/* These signals are used in the NORM display routine */

#define IRIMAGE_SIGNAL_CLEAR		7
#define IRIMAGE_SIGNAL_SCALE_UP		8
#define IRIMAGE_SIGNAL_SCALE_DOWN	9
#define IRIMAGE_SIGNAL_RECORD		10
#define IRIMAGE_SIGNAL_HOLD		11
#define IRIMAGE_SIGNAL_SEND		12
#define IRIMAGE_SIGNAL_OFFSET_UP_1	13
#define IRIMAGE_SIGNAL_OFFSET_DOWN_1	14
#define IRIMAGE_SIGNAL_OFFSET_UP_500	15
#define IRIMAGE_SIGNAL_OFFSET_DOWN_500	16
#define IRIMAGE_SIGNAL_TOGGLE_TRACK_SERVO 17
#define IRIMAGE_SIGNAL_MIN_WEIGHT_UP	18
#define IRIMAGE_SIGNAL_MIN_WEIGHT_DOWN	19
#define IRIMAGE_SIGNAL_MAX_ERROR_DOWN	20
#define IRIMAGE_SIGNAL_MAX_ERROR_UP	21
#define IRIMAGE_SIGNAL_GAIN_DOWN	22
#define IRIMAGE_SIGNAL_GAIN_UP		23
#define IRIMAGE_SIGNAL_TOGGLE_DIRECTION 24
#define IRIMAGE_SIGNAL_TARG_UP_1	25
#define IRIMAGE_SIGNAL_TARG_DN_1	26
#define IRIMAGE_SIGNAL_TARG_UP_10	27
#define IRIMAGE_SIGNAL_TARG_DN_10	28
#define IRIMAGE_SIGNAL_POS_UP_10	29
#define IRIMAGE_SIGNAL_POS_DN_10	30
#define IRIMAGE_SIGNAL_POS_UP_50	31
#define IRIMAGE_SIGNAL_POS_DN_50	32
#define IRIMAGE_SIGNAL_SIG_DIFF		33

/* This is used in several places */

#define IRIMAGE_SIGNAL_EXIT		50

/************************************************************************/
/* OPLE messages 400-499						*/
/************************************************************************/

#define FIRST_OPLE_MESSAGE		400

/* Client->Server Open ople socket, no data */

#define OPLE_OPEN_SOCKET		(FIRST_OPLE_MESSAGE+0)

/* Client->Server Close ople socket, no data */

#define OPLE_CLOSE_SOCKET		(FIRST_OPLE_MESSAGE+1)

/* 
 * Client-> Server select a target structure and data 
 * Server->Client response to what it just did
 */

struct s_select_target {
				int dl_number; 
				int target; 
				char set_active; 
		       };

#define OPLE_SELECT_TARGET		(FIRST_OPLE_MESSAGE+2)

/* 
 * Client->Server Set manual and oplen loop targets, data as in struct 
 * Note, the unsigned int should be OPLE_TIME. 
 * This is not cool but it works.
 */


struct s_target {
			double position; 
			double rate;
			int dl_number; 
			unsigned int time;
		};

#define OPLE_OPEN_LOOP_TARGET		(FIRST_OPLE_MESSAGE+3)
#define OPLE_MANUAL_TARGET		(FIRST_OPLE_MESSAGE+4)

/* Client->Server command to turn cart off, data is an int */

#define OPLE_CART_OFF			(FIRST_OPLE_MESSAGE+5)

/* Client->Server command to turn all carts off, no data */

#define OPLE_ALL_CARTS_OFF		(FIRST_OPLE_MESSAGE+6)

/* Client->Server command to send cart front, data is an int */

#define OPLE_CART_FRONT			(FIRST_OPLE_MESSAGE+7)

/* Client->Server command to send cart back, data is an int */

#define OPLE_CART_BACK			(FIRST_OPLE_MESSAGE+8)

/* Client->Server command to send cart track, data is an int */

#define OPLE_CART_TRACK			(FIRST_OPLE_MESSAGE+9)

/* Client->Server command to send cart home, data is an int */

#define OPLE_CART_HOME			(FIRST_OPLE_MESSAGE+10)

/* Client->Server command to clear cart laser, data is an int */

#define OPLE_CART_CLEAR_LASER		(FIRST_OPLE_MESSAGE+11)

/* Client->Server command to start astromod, no data */

#define OPLE_START_ASTROMOD		(FIRST_OPLE_MESSAGE+12)

/* Client->Server command to stop astromod, no data */

#define OPLE_STOP_ASTROMOD		(FIRST_OPLE_MESSAGE+13)

/* Client->Server command to set reference, data as in struct */

struct s_set_reference {
				double position;
				double rate;
		       };

#define OPLE_SET_REFERENCE_CART		(FIRST_OPLE_MESSAGE+14)

/* Client->Server request return messages of form SLECT_TARGET so
	client knows what's up */

#define OPLE_CURRENT_SELECT_TARGETS	(FIRST_OPLE_MESSAGE+15)

/* 
 * Client->Sever Request current manual target, no data.
 * Server->CLient Send current manual target, data is array of doubles.
 * Units are in meters.
 */

#define OPLE_CURRENT_MANUAL_TARGETS	(FIRST_OPLE_MESSAGE+16)

/* 
 * Client->Sever Request current open_loop targets, no data.
 * Server->CLient Send current open_loop targets, data is array of doubles.
 * Units are in meters.
 */

#define OPLE_CURRENT_OPEN_LOOP_TARGETS (FIRST_OPLE_MESSAGE+17)

/*
 * Client->Server Start scaning, data in structure determines
 * scan range frequency and so on. ANy client getting this message knows
 * not to try and do anything else for a while, at least it knows to pop
 * up the right window.
 * The STOP_SCAN message tells all the clients we are done.
 * Server->Client (mostly stop) to tell clients that we haved stopped.
 */

struct s_start_scan {
		double	start;
		double	stop;
		double	inc;
		double	lambda;
		double	freq;
		int	dl;
		int	scans;
	      };

#define	OPLE_START_SCAN			  (FIRST_OPLE_MESSAGE+18)
#define	OPLE_STOP_SCAN			  (FIRST_OPLE_MESSAGE+19)

/* 
 * Server-> Client Sends sscan status data. If the client get's this it then
 * knows that we're scanning
 */

struct s_scan_status {
		double	start;	 /* cm */
		double	stop;	 /* cm */
		double	current_position; /* cm */
		double	vel; /* um/S */
		double	scan_time_left;	/* Secs */
		int	dl;	/* cart */
		int	filenumber; /* Integer */
		int	current_scan;
		int	number_of_scans;
		     };
		
#define	OPLE_SCAN_STATUS		  (FIRST_OPLE_MESSAGE+20)

/* Client->Server Request to toggle metrology data, data integer cart number */

#define OPLE_METROLOGY_DATA_ON		  (FIRST_OPLE_MESSAGE+21)
#define OPLE_METROLOGY_DATA_OFF		  (FIRST_OPLE_MESSAGE+22)

/* Client->Server Request to sync time on ople rack, no data */

#define OPLE_SYNC_OPLE			  (FIRST_OPLE_MESSAGE+23)

/* Client->Server open/close log file, no data */

#define OPLE_OPEN_LOG			(FIRST_OPLE_MESSAGE+24)
#define OPLE_CLOSE_LOG			(FIRST_OPLE_MESSAGE+25)

/* 
 * Client->Server request sending same message to all clients, data is
 * an int to say which cart we wish to offset.
 * Server->Client Request opening offset window on client, data is
 * an int to say which cart we wish to offset.
 */

#define OPLE_RUN_OFFSET			(FIRST_OPLE_MESSAGE+26)

/*
 * Client->Server, request tracking errors, no data.
 * Server->Client, send tracking errors, data 2 arrays 
 * of floats[NUM_DELAY_LINES]. First cart positions, second cart errors.
 */

#define OPLE_TRACKING_ERROR		(FIRST_OPLE_MESSAGE+27)

/* 
 * Client->Server request for return message of same type, no data
 * Server->Client response. Data single char TRUE if astromod running,
 *		FALSE otherwise
 */

#define OPLE_ASTROMOD_RUNNING          (FIRST_OPLE_MESSAGE+28)

/* Message to set automatic reference on OPLE, data is a char TRUE or FALSE */

#define OPLE_AUTO_REFERENCE		(FIRST_OPLE_MESSAGE+29)

/*
 * Client->Server, request current status of limit and home switches.
 * Server->Client response. Data array[NUM_DELAY_LINES] of structures as below
 */

#define OPLE_LIMITS_STATUS		(FIRST_OPLE_MESSAGE+30)

struct s_limits_status {
				char	hard_front;
				char	soft_front;
				char	home;
				char	soft_back;
				char	hard_back;
			};

/*
 * Client->Server Request for message of same type, no data.
 * Server->Client Response, returns char array telling if each cart is
 * moving or not. TRUE means the cart is moving.
 */

#define OPLE_CARTS_MOVING		(FIRST_OPLE_MESSAGE+31)

/* 
 * Client->Server set new manual target offsets for all six scopes.
 * Data is array of NUM_SCOPE doubles with new offsets. Rates assumed
 * to be zero.
 */

#define OPLE_MANUAL_OFFSETS		(FIRST_OPLE_MESSAGE+32)

#define LAST_OPLE_MESSAGE		OPLE_MANUAL_OFFSETS

/************************************************************************/
/* And now we define the signals that OPLE knows about.			*/
/************************************************************************/

/* The first set of signals is for the scan mode */

#define OPLE_SIGNAL_SCAN_CENTER		1
#define OPLE_SIGNAL_SCAN_FORWARD	2
#define OPLE_SIGNAL_SCAN_BACKWARD	3
#define OPLE_SIGNAL_SCAN_INCREASE	4
#define OPLE_SIGNAL_SCAN_DECREASE	5
#define OPLE_SIGNAL_SCAN_STOP_ZERO	6
#define OPLE_SIGNAL_SCAN_STOP_HERE	7
#define OPLE_SIGNAL_SCAN_EDGE		8

/************************************************************************/
/* WOBBLE messages 500-599						*/
/************************************************************************/

#define FIRST_WOBBLE_MESSAGE	500

/*
 * Message to request BDISP. Client->Server. Data as per structure below.
 */

#define	WOBBLE_BDISP		(FIRST_WOBBLE_MESSAGE+0)

struct s_bdisp {
			int	display_length;
			char	display[256];
	       };

/*
 * CLient->Server reuests start of display_position function.
 * Data as in structure below.
 */

#define WOBBLE_CDISP		(FIRST_WOBBLE_MESSAGE+1)

struct s_cdisp {
			int	detector;
			int	length;
			char	display[256];
		};
/*
 * Server->CLient sending the cdisp status info as structure below.
 */

struct s_cdisp_status {
			float	a;
			float	b;
			float	c;
			float	d;
			float	sum;
			int	samples;
			char	name[81];
		      };

/*
 * Server->Client stop sending the cdisp status info.
 */

#define WOBBLE_CDISP_STOP	(FIRST_WOBBLE_MESSAGE+2)

/*
 * Client->Server request chanel names, no data 
 */

#define WOBBLE_NAMES           (FIRST_WOBBLE_MESSAGE+3)

/*
 * Server->Client list of names as per structure
 */

struct s_names {
		char names[2][81];
	       };

/* Client->Server - set names, data as per structure above */

#define WOBBLE_SET_NAMES	(FIRST_WOBBLE_MESSAGE+4)

/*
 * Client->server request status information on one chanel.
 * Data is an integer specifying the chanel.
 */

#define WOBBLE_STATUS		(FIRST_WOBBLE_MESSAGE+5)

/*
 * Server->Client status information, data in following structure
 */

struct s_det_status {
			int	      det;   /* Which detector */
			unsigned char mod_x; /* TRUE or FALSE */
			unsigned char mod_y; /* TRUE or FALSE */
			unsigned char on;    /* TRUE or FALSE */
			unsigned char send;  /* TRUE or FALSE... for sending */
		};

/*
 * CLient->Server turn signals on or off. No data
 */

#define WOBBLE_SIGON		(FIRST_WOBBLE_MESSAGE+6)
#define WOBBLE_SIGOFF		(FIRST_WOBBLE_MESSAGE+7)

/*
 * Client->server, for turing on/off servo, modulation and signals
 * all have integer (chanel) for arguement
 */

#define WOBBLE_ON		(FIRST_WOBBLE_MESSAGE+8)
#define WOBBLE_OFF		(FIRST_WOBBLE_MESSAGE+9)
#define WOBBLE_MODX		(FIRST_WOBBLE_MESSAGE+10)
#define WOBBLE_MODY		(FIRST_WOBBLE_MESSAGE+11)
#define WOBBLE_SENDON		(FIRST_WOBBLE_MESSAGE+12)
#define WOBBLE_SENDOFF		(FIRST_WOBBLE_MESSAGE+13)

/*
 * CLient->Server request reference frame images, data of form
 */

#define WOBBLE_REF		(FIRST_WOBBLE_MESSAGE+14)

struct s_ref {
		int	row;
		int	col;
		char	display[256];
	     };

/*
 * Server->Client for sending reference status information of form
 */

struct s_ref_status {
			float	frames_per_second;
			float	remote_frames_per_second;
			float	maximum;
			float	pos_x;
			float	pos_y;
			float 	org_x;
			float	org_y;
			float	err_x;
			float	err_y;
			int	frames;
			int	remote_frames;
			int	pix_x;
			int	pix_y;
			int	brightness;
			int	contrast;
			int 	zoom;
		    };

/*
 * Server->Client for stopping reference status information 
 */

#define WOBBLE_REF_STOP	(FIRST_WOBBLE_MESSAGE+15)

/*
 * Client->Server for setting sample period, data integer period
 */

#define WOBBLE_PERIOD		(FIRST_WOBBLE_MESSAGE+16)

/*
 * Client->Server for setting varriance period, data integer period
 */

#define WOBBLE_VAR_PERIOD	(FIRST_WOBBLE_MESSAGE+17)

/*
 * Client->Server for runnign auto-tuner, no data
 */

#define WOBBLE_AUTO_TUNE	(FIRST_WOBBLE_MESSAGE+18)

/*
 * Client->Server for setting current servo parameters
 * Server->Client for sending current servo parameters
 * Both have the following data structure
 */

#define WOBBLE_SERVO_PARAMS	(FIRST_WOBBLE_MESSAGE+19)

struct	s_servo_params {
				float	prop;
				float	diff;
				int	det;
		       };

/*
 * Client->Server, request servo params, data integer chanel number
 */

#define WOBBLE_GET_SERVO_PARAMS	(FIRST_WOBBLE_MESSAGE+20)

/*
 * Client->Server, request a display socket, data in structure s_socket_name
 */

#define WOBBLE_OPEN_DISPLAY_SOCKET	(FIRST_WOBBLE_MESSAGE+21)

/*
 * Client->Server for setting maximum allowed change. Data in structure.
 * Server->Client response to request for current value, same data.
 */

#define	WOBBLE_SET_MAX_CHANGE		(FIRST_WOBBLE_MESSAGE+22)

struct s_max_change {
			float	max_change;
			int	det;
		    };

/*
 * Client->Server, requesting current max. Data integer detector number
 */

#define	WOBBLE_GET_MAX_CHANGE		(FIRST_WOBBLE_MESSAGE+23)

/* Client->Server, sets names of scopes, data as per structure */

struct s_scope_names {
			char beam5_name[3];
			char beam6_name[3];
		     };

#define WOBBLE_SCOPE_NAMES		(FIRST_WOBBLE_MESSAGE+24)

/* 
 * Client->Server, request counts seen in system, no data.
 * Server->Client, return counts seen, data in structure.
 */

struct s_counts {
			int	beam5;
			int	beam6;
		};

#define WOBBLE_COUNTS			(FIRST_WOBBLE_MESSAGE+25)

/* 
 * Client->Server, request errors seen in system, no data.
 * Server->Client, return errors seen, data in structure.
 */

struct s_errors {
			float	beam5;
			float	beam6;
		};

#define WOBBLE_ERRORS			(FIRST_WOBBLE_MESSAGE+26)

/* Client->Server, requests logging of seeing, no data */

#define WOBBLE_LOG_SEEING		(FIRST_WOBBLE_MESSAGE+27)

/* Client->Server, open or close the log file, no data */

#define WOBBLE_OPEN_LOG			(FIRST_WOBBLE_MESSAGE+28)
#define WOBBLE_CLOSE_LOG		(FIRST_WOBBLE_MESSAGE+29)

/*
 * Client->Server, request a display socket, data in structure s_socket_name
 */

#define WOBBLE_OPEN_REF_SOCKET		(FIRST_WOBBLE_MESSAGE+30)

/* Client->Server start or stop data recording */

#define WOBBLE_START_RECORD		(FIRST_WOBBLE_MESSAGE+31)
#define WOBBLE_STOP_RECORD		(FIRST_WOBBLE_MESSAGE+32)

/*
 * Client->Server request channel spec frame images, data is same as WOBBLE_REF
 */

#define WOBBLE_CHAN			(FIRST_WOBBLE_MESSAGE+33)


/*
 * Server->Client for sending reference status information of form
 */

struct s_chan_status {
			float	frames_per_second;
			float	maximum;
			int	frames;
			int	pix_x;
			int	pix_y;
			int	brightness;
			int	contrast;
		    };

/*
 * Server->Client for stopping reference status information 
 */

#define WOBBLE_CHAN_STOP	(FIRST_WOBBLE_MESSAGE+34)

#define LAST_WOBBLE_MESSAGE		WOBBLE_CHAN_STOP

/************************************************************************/
/* signals, mostly used for within functions.				*/
/************************************************************************/

#define WOBBLE_SIGNAL_STOP		1
#define WOBBLE_SIGNAL_CLEAR		2
#define WOBBLE_SIGNAL_BRIGHT_UP		3
#define WOBBLE_SIGNAL_BRIGHT_DOWN	4
#define WOBBLE_SIGNAL_CONTRAST_UP	5
#define WOBBLE_SIGNAL_CONTRAST_DOWN	6
#define WOBBLE_SIGNAL_REVERSE_COLOR	7
#define WOBBLE_SIGNAL_SET_ORIGIN	8
#define WOBBLE_SIGNAL_ZOOM_IN		9
#define WOBBLE_SIGNAL_ZOOM_OUT		10
#define WOBBLE_SIGNAL_CROSS_HAIR	11
#define WOBBLE_SIGNAL_AUTOALIGN		12
#define WOBBLE_SIGNAL_UNDOALIGN		13
#define WOBBLE_SIGNAL_BIAS		14

/************************************************************************/
/* ESP messages 600-699							*/
/************************************************************************/

#define FIRST_ESP_MESSAGE	600

/* Structure for axis information */

struct AXIS{
                double position;
                double relative_step_size;
                double real_position;
                double velocity;
                double acceleration;
                float min;
                float max;
                int num;
                int beam;
                int power;
                int motion_done;
           };

/* 
 * The ON OFF message have one byte of data to say which port. 
 * These messages only sent from client to server.
 */

#define ESP_ON			(FIRST_ESP_MESSAGE+0)
#define ESP_OFF			(FIRST_ESP_MESSAGE+1)

/* The GET_STATUS goes from client to server and has no data. */

#define ESP_GET_STATUS		(FIRST_ESP_MESSAGE+2)

/* The STATUS goes from server to client and has one byte as data . */

#define ESP_STATUS		(FIRST_ESP_MESSAGE+3)

/* GET_NAMES goes from client to server and has no data */

#define ESP_GET_NAMES		(FIRST_ESP_MESSAGE+4)

/* NAMES goes from server to client and has the names as data */

#define ESP_NAMES		(FIRST_ESP_MESSAGE+5)

/* MOVE_ABS goes from server to client and has the names as data */

#define ESP_MOVE_ABS		(FIRST_ESP_MESSAGE+6)

/* MOVE_HOME goes from server to client and has the names as data */

#define ESP_MOVE_HOME		(FIRST_ESP_MESSAGE+7)

/* ESP_TEXT goes from server to client and has the commands as data */

#define ESP_TEXT		(FIRST_ESP_MESSAGE+8)

/* MOVE_REL goes from server to client and has the commands as data */

#define ESP_MOVE_REL		(FIRST_ESP_MESSAGE+9)

/* The ESP_GET_POWER_STATUS goes from client to server and has no data */

#define ESP_GET_POWER_STATUS	(FIRST_ESP_MESSAGE+10)

/* The ESP_POWER_STATUS goes from server to client and has data */

#define ESP_POWER_STATUS	(FIRST_ESP_MESSAGE+11)

/* The ESP_POSITION goes from server to client and has data */

#define ESP_POSITION		(FIRST_ESP_MESSAGE+12)

/* The ESP_GET_POSITION goes from server to client and has data */

#define ESP_GET_POSITION	(FIRST_ESP_MESSAGE+13)

/* The ESP_MOTION_DONE_STATUS goes from server to client and has data */

#define ESP_MOTION_DONE_STATUS	(FIRST_ESP_MESSAGE+14)

/* The ESP_GET_MOTION_DONE_STATUS goes from server to client and has data */

#define ESP_GET_MOTION_DONE_STATUS	(FIRST_ESP_MESSAGE+15)

/* The ESP_SET_VELOCITY goes from server to client and has data */

#define ESP_SET_VELOCITY	(FIRST_ESP_MESSAGE+16)

/* The ESP_SET_ACCELERATION goes from server to client and has data */

#define ESP_SET_ACCELERATION	(FIRST_ESP_MESSAGE+17)

/* The ESP_GET_MINMAX goes from server to client and has data */

#define ESP_GET_MINMAX		(FIRST_ESP_MESSAGE+18)

/* The ESP_MINMAX goes from server to client and has data */

#define ESP_MINMAX		(FIRST_ESP_MESSAGE+19)

/* The ESP_GET_VELOCITY goes from server to client and has data */

#define ESP_GET_VELOCITY	(FIRST_ESP_MESSAGE+20)

/* The ESP_GET_ACCELERATION goes from server to client and has data */

#define ESP_GET_ACCELERATION	(FIRST_ESP_MESSAGE+21)

/* The ESP_VELOCITY goes from server to client and has data */

#define ESP_VELOCITY		(FIRST_ESP_MESSAGE+22)

/* The ESP_ACCELERATION goes from server to client and has data */

#define ESP_ACCELERATION	(FIRST_ESP_MESSAGE+23)

/* The ESP_SET_BEAM goes from client to server and has data */

#define ESP_SET_BEAM		(FIRST_ESP_MESSAGE+24)


#define LAST_ESP_MESSAGE	ESP_SET_BEAM

/************************************************************************/
/* METLAS messages 700-799						*/
/************************************************************************/

#define FIRST_METLAS_MESSAGE	700

/* 
 * The ON OFF message have one byte of data to say which port. 
 * These messages only sent from client to server.
 */

#define METLAS_ON		(FIRST_METLAS_MESSAGE+0)
#define METLAS_OFF		(FIRST_METLAS_MESSAGE+1)

/* The GET_POWER goes from client to server and has no data. */

#define METLAS_GET_POWER	(FIRST_METLAS_MESSAGE+2)

/* The GET_DIODE command asks if the diode is on or off */

#define METLAS_GET_DIODE	(FIRST_METLAS_MESSAGE+3)

/* The METLAS_FLUSH_SERIAL command askes server to clear the serial buffer */

#define METLAS_FLUSH_SERIAL	(FIRST_METLAS_MESSAGE+4)

/* POWER goes from server to client and has the amount of laser power in it */

#define METLAS_POWER		(FIRST_METLAS_MESSAGE+5)

/* DIODE goes from server to client and sends the state of the diode */

#define METLAS_DIODE		(FIRST_METLAS_MESSAGE+6)

#define LAST_METLAS_MESSAGE	METLAS_DIODE

/************************************************************************/
/* METROL messages 800-899						*/
/************************************************************************/

#define FIRST_METROL_MESSAGE	800

/* Client->Server request to open metroloy port, no data */

#define METROL_MESSAGE_OPEN_PORT	(FIRST_METROL_MESSAGE+0)

/* 
 * Client->Server requests a display to start data is display name 
 * Server->Client tells client we are displaying thing, no data.
 */

#define METROL_MESSAGE_START_DISPLAY	(FIRST_METROL_MESSAGE+1)

/*
 * Server->Client tells client we have stopped display, no data
 */ 

#define METROL_MESSAGE_STOP_DISPLAY	(FIRST_METROL_MESSAGE+2)

/* Client->Server/Server->Client, Channels to display. Data of type */

struct s_metrol_channels {
				int	chan1;
				int	chan2;
			};

#define METROL_MESSAGE_SELECT_CHANNELS		(FIRST_METROL_MESSAGE+3)

/* Client->Server Reuest for message of type METROL_MESSAGE_SELECT_CHANNELS */

#define METROL_MESSAGE_WHICH_CHANNELS		(FIRST_METROL_MESSAGE+4)

/* Client->Server, request to change scale. Int Data one of the signals below */

#define METROL_MESSAGE_CHANGE_SCALE		(FIRST_METROL_MESSAGE+5)

/* Server->Client, current volts and time per cm, data of struct below */

struct s_metrol_scale {
				float volts_per_div;
				float seconds_per_div;
			};

#define METROL_MESSAGE_SCALE		(FIRST_METROL_MESSAGE+6)

/* 
 * Client->Server requests a display to start data is display name 
 * Data following structure.
 * Server->Client tells client we are displaying thing, no data.
 */

struct s_metrol_mult {
			int active[12];
			char display[256];
		     };

#define METROL_MESSAGE_START_MULT	(FIRST_METROL_MESSAGE+7)

/*
 * Server->Client tells client we have stopped display, no data
 */ 

#define METROL_MESSAGE_STOP_MULT	(FIRST_METROL_MESSAGE+8)

#define LAST_METROL_MESSAGE			METROL_MESSAGE_SCALE

/************************************************************************/
/* signals, mostly used for within functions.				*/
/************************************************************************/

#define METROL_SIGNAL_STOP		1
#define METROL_SIGNAL_VUP		2
#define METROL_SIGNAL_VDN		3
#define METROL_SIGNAL_HUP		4
#define METROL_SIGNAL_HDN		5
#define METROL_SIGNAL_TOGGLE_SHOW_PS	6
#define METROL_SIGNAL_CLEAR_FAULT	7

/************************************************************************/
/* PICO messages 900-999						*/
/************************************************************************/

#define FIRST_PICO_MESSAGE	900

/*
 * The first message is one to get the names and types of each mirror 
 * THis message goes from client to server and should initiate a 
 * PICO_MIRROR_LIST message in return.
 */

#define PICO_GET_MIRROR_LIST	(FIRST_PICO_MESSAGE+0)

/*
 * This is the response message form the server for the above message.
 * It's should contain a mirror_list structure.
 */

#define MAX_MIRRORS	60

typedef struct smirror {

	int  axis_one;
	int  axis_two;
	int  pulses;
	char type;	/* COUPLED, UNCOUPLED or SINGLE */
	char name[81];

	} smirror;

struct SMIRROR_LIST {

	smirror mirrors[MAX_MIRRORS];
	int	n;		/* How many mirrors are there? */
	char	name[81];	/* Name of this server */

	};

#define PICO_MIRROR_LIST	(FIRST_PICO_MESSAGE+1)

/*
 * This message goes from CLIENT to SERVER and is a request to move a mirror
 * in a certain direction by a certain amount.
 * It should contain a morror_move structure that holds the relevant data.
 */

#define UP		0
#define DOWN		1
#define LEFT		2
#define RIGHT		3
#define FORWARD		4
#define BACKWARD	5
#define	COUPLED		6
#define UNCOUPLED	7
#define SINGLE		8

typedef struct smirror_move {

	int	mirror;		/* Which mirror? */
	int	direction;	/* Which way ? */
	int	pulses;		/* How far? */

	} mirror_move;

#define PICO_MOVE_MIRROR	(FIRST_PICO_MESSAGE+2)

/*
 * This message goes from CLIENT to SERVER and asks it to re-initialize the
 * hardware. The ID of the device is sent back.
 */

#define PICO_INIT		(FIRST_PICO_MESSAGE+3)

#define LAST_PICO_MESSAGE	PICO_INIT

/************************************************************************/
/* RPC messages 1000-1099						*/
/************************************************************************/

#define FIRST_RPC_MESSAGE	1000

/* 
 * The ON OFF message have one byte of data to say which port. 
 * These messages only sent from client to server.
 */

#define RPC_ON		(FIRST_RPC_MESSAGE+0)
#define RPC_OFF		(FIRST_RPC_MESSAGE+1)

/* The GET_STATUS goes from client to server and has no data. */

#define RPC_GET_STATUS	(FIRST_RPC_MESSAGE+2)

/* The STATUS goes from server to client and has one byte as data . */

#define RPC_STATUS	(FIRST_RPC_MESSAGE+3)

/* GET_NAMES goes from client to server and has no data */

#define RPC_GET_NAMES	(FIRST_RPC_MESSAGE+4)

/* goes from server to client and has the names as a set of strings as data */

#define RPC_NAMES	(FIRST_RPC_MESSAGE+5)

#define LAST_RPC_MESSAGE	RPC_NAMES

/************************************************************************/
/* SHUTTER messages 1100-1199						*/
/************************************************************************/

#define FIRST_SHUTTER_MESSAGE	1100

/* Open a shutter, argument single byte */

#define OPEN_SHUTTER		(FIRST_SHUTTER_MESSAGE+0)

/* Close a shutter, argument single byte */

#define CLOSE_SHUTTER		(FIRST_SHUTTER_MESSAGE+1)

/* Request or send shutter status, data as structure */

struct sshutter_status {
	unsigned char	shutter;
	unsigned char	status;
		       };
#define STATUS_SHUTTER		(FIRST_SHUTTER_MESSAGE+2)

/* Request or send configuration, data as structure */

#define MAX_CONFIGS	64
#ifndef NUMBER_SHUTTERS
#define NUMBER_SHUTTERS 16
#endif

extern struct sshutter_conf {
	int  number_confs;
	char name[81];
	char shutter_names[NUMBER_SHUTTERS][81];
	char conf_names[MAX_CONFIGS][81];
	unsigned char conf[MAX_CONFIGS][NUMBER_SHUTTERS];
		     } shutter_conf;

#define SHUTTER_CONF		(FIRST_SHUTTER_MESSAGE+3)

#define LAST_SHUTTER_MESSAGE	SHUTTER_CONF

/************************************************************************/
/* TELESCOPE messages 1200-1299						*/
/************************************************************************/

#define FIRST_TELESCOPE_MESSAGE	1200

/*
 * Client->Server, request a display socket, data in structure s_socket_name
 */

#define TELESCOPE_OPEN_DISPLAY_SOCKET	(FIRST_TELESCOPE_MESSAGE+0)

/* 
 * Client->Server, to put acqusition in/out or change to finder.
 * ACQ_IN takes data as a NULL terminated string. NO data for the others.
 */

#define TELESCOPE_ACQ_IN		(FIRST_TELESCOPE_MESSAGE+1)
#define TELESCOPE_ACQ_OUT		(FIRST_TELESCOPE_MESSAGE+2)
#define TELESCOPE_FINDER_IN		(FIRST_TELESCOPE_MESSAGE+3)
#define TELESCOPE_SPY1_IN		(FIRST_TELESCOPE_MESSAGE+4)
#define TELESCOPE_SPY2_IN		(FIRST_TELESCOPE_MESSAGE+5)
#define TELESCOPE_M10ALIGN_IN		(FIRST_TELESCOPE_MESSAGE+6)
#define TELESCOPE_CH6_IN		(FIRST_TELESCOPE_MESSAGE+7)
#define TELESCOPE_CH7_IN		(FIRST_TELESCOPE_MESSAGE+8)
#define TELESCOPE_CH8_IN		(FIRST_TELESCOPE_MESSAGE+9)

/* 
 * Client->Server, turn TV on and off and control bounding box.
 * TV_ON takes data as a NULL terminated string. NO data for the others.
 */

#define TELESCOPE_TV_ON			(FIRST_TELESCOPE_MESSAGE+10)
#define TELESCOPE_TV_OFF		(FIRST_TELESCOPE_MESSAGE+11)
#define TELESCOPE_TV_SBB		(FIRST_TELESCOPE_MESSAGE+12)
#define TELESCOPE_TV_UBB		(FIRST_TELESCOPE_MESSAGE+13)

/*
 * Messages for adjusting Brightness/contrast of TV. All Client-Server.
 * ALl of these induce the Server to send a TELESCOPE_BRIGHTNESS_CONTRAST
 * message with current brightness and contrast information.
 * Client->Server TELESCOPE_BRIGHTNESS_CONTRAST induces server to
 * just send brightness/contrast data if there is no data. If there is
 * data it is used to actually set the brightness and contrast.
 */

#define TELESCOPE_TV_BRIGHTNESS_UP	(FIRST_TELESCOPE_MESSAGE+14)
#define TELESCOPE_TV_BRIGHTNESS_DOWN	(FIRST_TELESCOPE_MESSAGE+15)
#define TELESCOPE_TV_CONTRAST_UP	(FIRST_TELESCOPE_MESSAGE+16)
#define TELESCOPE_TV_CONTRAST_DOWN	(FIRST_TELESCOPE_MESSAGE+17)

struct s_brightness_contrast {
	int brightness;
	int contrast;
			     };

#define TELESCOPE_TV_BRIGHTNESS_CONTRAST (FIRST_TELESCOPE_MESSAGE+18)

/*
 * Messages for controling the tracking origin. First two are both
 * Client->Server with no data.
 */

#define TELESCOPE_SET_ORIGIN		 (FIRST_TELESCOPE_MESSAGE+19)
#define TELESCOPE_CENTER_ORIGIN		 (FIRST_TELESCOPE_MESSAGE+20)

/* 
 * Next one is Client->Server requesting change in X or Y axis origin, data
 * is in form of an integer that defines the size of the change.
 */

#define TELESCOPE_X_ORIGIN_CHANGE	(FIRST_TELESCOPE_MESSAGE+21)
#define TELESCOPE_Y_ORIGIN_CHANGE	(FIRST_TELESCOPE_MESSAGE+22)

/*
 * Client->Server for turing tracking on and off. No data.
 */

#define TELESCOPE_TRACK_ON		(FIRST_TELESCOPE_MESSAGE+23)
#define TELESCOPE_TRACK_OFF		(FIRST_TELESCOPE_MESSAGE+24)

/* 
 * Client->Server
 * Messages for turning data receive on, off and for clearing lost count.
 */

#define TELESCOPE_REC_ON		(FIRST_TELESCOPE_MESSAGE+25)
#define TELESCOPE_REC_OFF		(FIRST_TELESCOPE_MESSAGE+26)

/* Client->Server set tip/tilt position to zero. No data */

#define TELESCOPE_TT_ZERO		(FIRST_TELESCOPE_MESSAGE+27)

/* Client->Server Set servo type, data one unsigned character */

#ifndef NO_SERVO
#define NO_SERVO                0 /* NOTE ALSO USED AS BITS!!! */
#define WOBBLER_SERVO           1
#define ACQUISITION_SERVO       2
#define BOTH_SERVO              3
#define FINDER_SERVO            4
#endif

#define TELESCOPE_SERVO			(FIRST_TELESCOPE_MESSAGE+28)

/* Client->Server run grab command. No data. */

#define TELESCOPE_GRAB			(FIRST_TELESCOPE_MESSAGE+29)

/* Client->Server Dis/En-able telescope motion, no data */

#define TELESCOPE_DISABLE		(FIRST_TELESCOPE_MESSAGE+30)
#define TELESCOPE_ENABLE		(FIRST_TELESCOPE_MESSAGE+31)

/* Client->Server time sync commands, no data */

#define TELESCOPE_TSYNC			(FIRST_TELESCOPE_MESSAGE+32)
#define TELESCOPE_TDATE			(FIRST_TELESCOPE_MESSAGE+33)

/* Client->Server Stow and xoffzero, no data */

#define TELESCOPE_STOW			(FIRST_TELESCOPE_MESSAGE+34)
#define TELESCOPE_XOFFZERO		(FIRST_TELESCOPE_MESSAGE+35)

/* Client-> Autodome control, data single char ON and OFF */

#ifndef ON
#define ON 1
#define OFF 0
#endif

#define TELESCOPE_AUTODOME		(FIRST_TELESCOPE_MESSAGE+36)

/* Client->Server Send dome to home, no data */

#define TELESCOPE_HOMEDOME		(FIRST_TELESCOPE_MESSAGE+37)

/* Client->Server Mirror cover open/close, data single char OPEN/CLOSE */

#ifndef OPEN_COVER
#define OPEN_COVER 0
#define CLOSE_COVER 1
#endif 

#define TELESCOPE_PRIMARY_COVER		(FIRST_TELESCOPE_MESSAGE+38)
#define TELESCOPE_M3_COVER		(FIRST_TELESCOPE_MESSAGE+39)
#define TELESCOPE_FINDER_COVER		(FIRST_TELESCOPE_MESSAGE+40)
#define TELESCOPE_M5_COVER		(FIRST_TELESCOPE_MESSAGE+41)
#define TELESCOPE_DOME_SLIT		(FIRST_TELESCOPE_MESSAGE+42)

/*
 * Client->Server Set EL/AZ for telescope, data in structure below 
 */

struct s_elaz {
		double	el; /* In deciaml degrees! */
		double  az; /* In deciaml degrees! */
	      };

#define TELESCOPE_ELAZ	 	  	(FIRST_TELESCOPE_MESSAGE+43)

/* Client->Server Set RA/DEC for telescope, data in structure below */

struct s_radec {
		double	ra; /* In radians! */
		double  dec; /* In radians! */
	      };

#define TELESCOPE_RADEC	 	  	(FIRST_TELESCOPE_MESSAGE+44)

/* Client->Server, Set HR# of object to move to, data an int */

#define TELESCOPE_HR	 	  	(FIRST_TELESCOPE_MESSAGE+45)

/* Client->Server Send scope to next position, no data */

#define TELESCOPE_NEXT			(FIRST_TELESCOPE_MESSAGE+46)

/* Client->Server, change paddle setting, argument structure */

#ifndef PAD_STOP
#define PAD_STOP                0
#define PAD_UP                  1
#define PAD_DOWN                2
#define PAD_EAST                3
#define PAD_WEST                4
#endif

struct s_paddle {
		float rate;
		char direction;
		};

#define TELESCOPE_PADDLE		(FIRST_TELESCOPE_MESSAGE+47)

/* Client->Server, request to flush com port, no data */

#define TELESCOPE_FLUSH_COM_PORT	(FIRST_TELESCOPE_MESSAGE+48)

/*
 * Client->Server, request receive rate, no data
 * Server->Client, return receive rate, data int
 */

#define TELESCOPE_RECEIVE_RATE		(FIRST_TELESCOPE_MESSAGE+49)

/* Client->Server turns TCS rotation stuff ON or OFF, no data. */

#define TELESCOPE_START_TCS		(FIRST_TELESCOPE_MESSAGE+50)
#define TELESCOPE_STOP_TCS		(FIRST_TELESCOPE_MESSAGE+51)

/* 
 * Client->Server, request current telescope delta, no data.
 * Server->Client return current telescope delta, data float.
 */

#define TELESCOPE_DELTA			(FIRST_TELESCOPE_MESSAGE+52)

/*
 * Client->telescope, request check for star, no data.
 * Server->Client, response. Data char, TRUE if star is there.
 */

#define TELESCOPE_CHECK_FOR_STAR	(FIRST_TELESCOPE_MESSAGE+53)

/* 
 * Client-Serevr, request acqusition tracking error, no data
 * Server->Client, return acqusition tracking error, data float
 */

#define TELESCOPE_CHECK_FOR_LOCK	(FIRST_TELESCOPE_MESSAGE+54)

/* Client->Server request initialization of TCS, no data */

#define TELESCOPE_TCS_INIT		(FIRST_TELESCOPE_MESSAGE+55)

/* Client->Server Request TEMA timeout clear. */

#define TELESCOPE_TEMA_CLEAR_TIMEOUT	(FIRST_TELESCOPE_MESSAGE+56)

/* Messages for controlling TV zoom, no data, client->server only. */

#define TELESCOPE_TV_ZOOM_IN		(FIRST_TELESCOPE_MESSAGE+57)
#define TELESCOPE_TV_ZOOM_OUT		(FIRST_TELESCOPE_MESSAGE+58)
#define TELESCOPE_TV_ZOOM_LEFT		(FIRST_TELESCOPE_MESSAGE+59)
#define TELESCOPE_TV_ZOOM_RIGHT		(FIRST_TELESCOPE_MESSAGE+60)
#define TELESCOPE_TV_ZOOM_UP		(FIRST_TELESCOPE_MESSAGE+61)
#define TELESCOPE_TV_ZOOM_DOWN		(FIRST_TELESCOPE_MESSAGE+62)

/* CLient->Server, a request for message type TELESCOPE_SET_TV_MEAN_N */

#define TELESCOPE_GET_TV_MEAN_N		(FIRST_TELESCOPE_MESSAGE+63)

/* Client->Server, or Server->CLient set tv mean n, data an unsigned char */

#define TELESCOPE_SET_TV_MEAN_N		(FIRST_TELESCOPE_MESSAGE+64)

/* Client->Server for going into TEMA local mode */

#define TELESCOPE_TEMA_LOCAL		(FIRST_TELESCOPE_MESSAGE+65)

/* Client->Server for collection of pointing model data */

#define TELESCOPE_OPEN_POINT_DATA	(FIRST_TELESCOPE_MESSAGE+66)
#define TELESCOPE_CLOSE_POINT_DATA	(FIRST_TELESCOPE_MESSAGE+67)
#define TELESCOPE_COLLECT_POINT_DATA	(FIRST_TELESCOPE_MESSAGE+68)

/* Client->Server FINDSTAR for telescope, data in structure below */

#define TELESCOPE_FINDSTAR		(FIRST_TELESCOPE_MESSAGE+69)

struct s_findstar {
		float	el; /* In deciaml degrees! */
		float  az; /* In deciaml degrees! */
		float   mag; /* Minimum magnitude */
	      };

/* 
 * Client->Server request telescope position, no data 
 * Server->Client return of scope position, data as struct below.
 */

#define TELESCOPE_POSITION		(FIRST_TELESCOPE_MESSAGE+70)

struct s_scope_pos {
		double	el; /* In decimal degrees! */
		double  az; /* In decimal degrees! */
		double	ra; /* In radians! */
		double  dec; /* In radians! */
	      };

/* Client->Server, request aquire function start going */

#define TELESCOPE_ACQUIRE		(FIRST_TELESCOPE_MESSAGE+71)

/* Client->Server, controls heaters at TEMA, data char TRUE for ON. */

#define TELESCOPE_DEHUMID_HEAT		(FIRST_TELESCOPE_MESSAGE+72)

/* 
 * CLient Server, request status of covers. no data.
 * Server->Client, returns result of status, data as struct below.
 */

#define TELESCOPE_TEMA_STATUS		(FIRST_TELESCOPE_MESSAGE+73)

struct s_tema_status {
			int	primary;
			int	m3;
			int 	finder;
			int 	m5;
			int	acq;
		     };

#define LAST_TELESCOPE_MESSAGE		TELESCOPE_TEMA_STATUS

/************************************************************************/
/* And now we define the signals that TELESCOPE knows about.		*/
/************************************************************************/

#define TELESCOPE_SIGNAL_EXIT		1

/************************************************************************/
/* CENTRAL_SCRUTINIZER messages 1300-1399				*/
/************************************************************************/

#define FIRST_CENTRAL_SCRUTINIZER_MESSAGE	1300

/* 
 * Server->Client A message for the status window and log file. Normally
 * onyl meant for the central_scrutinizer client. All other clients should 
 * simply ignore this message. Data the string itself, not including
 * terminating NULL.
 */

#define CENTRAL_SCRUTINIZER_LOG		(FIRST_CENTRAL_SCRUTINIZER_MESSAGE+0)

#define LAST_CENTRAL_SCRUTINIZER_MESSAGE	CENTRAL_SCRUTINIZER_LOG

/************************************************************************/
/* SOCKET_MANAGER messages 1400-1499					*/
/************************************************************************/

#define FIRST_SOCKET_MANAGER			1400

/*
 * Client->Server, Request for a process's socket info. Data is a string
 * representing the name of the process you are interested in.
 */

#define SOCKET_MANAGER_GET_PROCESS		(FIRST_SOCKET_MANAGER+0)

/* 
 * Client->Server Request to register a process's socket numbers
 * data in form of structure.
 * Server->Client Response to SOCKET_MANAGER_GET_PROCESS
 */

struct s_process_sockets {
			int	pid;		/* PID of server process */
			int	commands;	/* Command port */
			int	data;		/* Data port */
			int	messages;	/* Message port */
			char	restart_command[256]; /* Guess */
			char	name[80];	/* Process name */
			char	machine[80];	/* Computer name */
		 };

#define SOCKET_MANAGER_SET_PROCESS		(FIRST_SOCKET_MANAGER+1)

/*
 * Client->Server, Request to remove process from the list. Data string
 * representing the name of the process you are interested in.
 * Server should reposnd with a SOCKET_MANAGER_SET_PROCESS message
 * with all socket numbers set to -1.
 */

#define SOCKET_MANAGER_REMOVE_PROCESS		(FIRST_SOCKET_MANAGER+2)

/*
 * Client->Server, Request a list of all process names. No data.
 * Server->Client, Returns list. Data is one long NULL terminated string
 *                 containing a list of '\n' separated names.
 */

#define SOCKET_MANAGER_LIST_NAMES		(FIRST_SOCKET_MANAGER+3)

/* Server->Client, lets client know no such process exists. No data */

#define SOCKET_MANAGER_NO_SUCH_PROCESS		(FIRST_SOCKET_MANAGER+4)

#define LAST_SOCKET_MANAGER			(SOCKET_MANAGER_NO_SUCH_PROCESS)

/************************************************************************/
/* POP_SERVER messages 1500-1599					*/
/************************************************************************/

#define FIRST_POP_MESSAGE			1500
/* client to server */
#define POP_IN					FIRST_POP_MESSAGE+0
/* client to server */
#define POP_OUT					FIRST_POP_MESSAGE+1
/* client to server */
#define POP_OFF					FIRST_POP_MESSAGE+2
/* client to server */
#define POP_LED_RED				FIRST_POP_MESSAGE+3
/* client to server */
#define POP_LED_GREEN				FIRST_POP_MESSAGE+4
/* client to server */
#define POP_LED_YELLOW				FIRST_POP_MESSAGE+5
/* client to server */
#define POP_LED_OFF				FIRST_POP_MESSAGE+6
/* client to server */
#define POP_SEND_STATUS				FIRST_POP_MESSAGE+7
/* server to client */
#define POP_STATUS				FIRST_POP_MESSAGE+8
/* that's all folks */
#define LAST_POP_MESSAGE			POP_STATUS

/************************************************************************/
/* WEATHER_SERVER messages 1600-1699					*/
/************************************************************************/

#define FIRST_WEATHER_MESSAGE			1600

/* Server to client, for pushing the current conditions */

#define WEATHER_NOW				(FIRST_WEATHER_MESSAGE+0)

struct s_weather_data {
			float hour_ut;		/* Time in fractional hours */
			float wind_speed;
			float wind_dir;
			float gust;
			float gust_dir;
			float outdoor_temp;
			float indoor_temp;
			float outdoor_humid;
			float dew_point;
			float wind_speed_avg;
		      };

/*
 * Client-Server, request current data file name.
 * Server->Client, response, data is the filename not 
 * including terminating null.
 */

#define WEATHER_FILE_NAME			(FIRST_WEATHER_MESSAGE+1)

#define LAST_WEATHER_MESSAGE			WEATHER_FILE_NAME

/************************************************************************/
/* THORLABS_FW102 messages 1700-1799					*/
/************************************************************************/

#define FIRST_THORLABS_FW102_MESSAGE			1700

/*
 * Client->Server Request filter names list no data
 * Server->Client Send name list, data in structure below
 */

#define THORLABS_FW102_NAMES			(FIRST_THORLABS_FW102_MESSAGE+0)

struct s_thorlabs_fw102_names {
		char	unit_name[81];
		char	filter_names[6][81];
			      };

/*
 * Client->Server Request new filter, data char, new filter
 */

#define THORLABS_FW102_SET_FILTER		(FIRST_THORLABS_FW102_MESSAGE+1)

/*
 * Client->Server Request unit ID, no data
 */

#define THORLABS_FW102_IDN			(FIRST_THORLABS_FW102_MESSAGE+2)

/*
 * Client->Server Request current filter, no data
 * Server->Client Reply, data char=current filter
 */

#define THORLABS_FW102_CURRENT_FILTER		(FIRST_THORLABS_FW102_MESSAGE+3)

/* Client->Server Reuest a reopening of the serial port. */

#define THORLABS_FW102_REOPEN_PORT		(FIRST_THORLABS_FW102_MESSAGE+4)

#define LAST_THORLABS_FW102_MESSAGE		THORLABS_FW102_REOPEN_PORT

/************************************************************************/
/* FAST_CLOCK messages 1800-1899					*/
/************************************************************************/

#define FIRST_FAST_LOCK_MESSAGE			1800

/* Client->Server, clears lost ticks. No data. */

#define FAST_CLOCK_CLEAR_LOST_TICKS		(FIRST_FAST_LOCK_MESSAGE)

/* 
 * Client->Server, Initiate a fast clock calibration, 
 * data unsigned char number of seconds.
 */

#define FAST_CLOCK_CALIBRATE			(FIRST_FAST_LOCK_MESSAGE+1)

/*
 * Server->Client sending information about the calibration process.
 * Data unsigned char, number of seconds left.
 */

#define FAST_CLOCK_CALIBRATE_INFO		(FIRST_FAST_LOCK_MESSAGE+2)

/* Client->Server For setting the clock period. Data integer. */

#define FAST_CLOCK_SET_PERIOD			(FIRST_FAST_LOCK_MESSAGE+3)

/* Client->Server, request the curretn rate. No data */

#define FAST_CLOCK_GET_RATE			(FIRST_FAST_LOCK_MESSAGE+4)

/* 
 * Client->Server For setting the clock rate. Data integer.
 * Server->Client as a response to FAST_CLOCK_GET_RATE. Data integer.
 */

#define FAST_CLOCK_SET_RATE			(FIRST_FAST_LOCK_MESSAGE+5)

/* 
 * Client->Server, request the time. No data.
 * Server->Client, sending the time, Data fast_clock_time_struct.
 */

#ifndef __CLOCK__
#ifndef __CLOCK_CLISERV__
#define CHARA_TIME int
#endif
#endif
#ifndef FAST_CLOCK_TIME_STRUCT_DEFINED
typedef struct FAST_CLOCK_TIME_STRUCT
        {
                CHARA_TIME time_now;
                CHARA_TIME lost_ticks;
                                                                                
        } fast_clock_time_struct;
#define FAST_CLOCK_TIME_STRUCT_DEFINED
#endif

#define FAST_CLOCK_TIME                     	(FIRST_FAST_LOCK_MESSAGE+6)

/* Client->Server to force a sync of the fast clock to the chara clock */

#define FAST_CLOCK_SYNC                     	(FIRST_FAST_LOCK_MESSAGE+7)

#define LAST_FAST_CLOCK_MESSAGE			FAST_CLOCK_SYNC

/************************************************************************/
/* signals, mostly used for within functions.				*/
/************************************************************************/

#define FAST_CLOCK_SIGNAL_STOP		1

/************************************************************************/
/* TARGET messages 1900-1999                                            */
/************************************************************************/

#define FIRST_TARGET_MESSAGE                    1900

/* Client->Server */
#define TARGET_UP                               (FIRST_TARGET_MESSAGE)

/* Client->Server */
#define TARGET_DOWN                             (FIRST_TARGET_MESSAGE+1)

/* Client->Server */
#define TARGET_LED_ON                           (FIRST_TARGET_MESSAGE+2)

/* Client->Server */
#define TARGET_LED_OFF                          (FIRST_TARGET_MESSAGE+3)

/* Client->Server */
#define TARGET_RESET                            (FIRST_TARGET_MESSAGE+4)

/* Client->Server */
#define TARGET_REMOTE                           (FIRST_TARGET_MESSAGE+5)

/* Client->Server */
#define TARGET_GET_STATUS                       (FIRST_TARGET_MESSAGE+6)

/* Server->Client */
#define TARGET_STATUS                           (FIRST_TARGET_MESSAGE+7)

#define LAST_TARGET_MESSAGE                     (TARGET_STATUS)

/************************************************************************/
/* TIPTILT messages 2000-2099                                           */
/************************************************************************/

#define FIRST_TIPTILT_MESSAGE                    2000

/*
 * Client->Server for setting exposure time, data integer exposure time mS
 */

#define TIPTILT_EXPOSURE		(FIRST_TIPTILT_MESSAGE+0)

/*
 * Client->Server for setting things to FULL FRAME mode. No data
 */

#define TIPTILT_FULL_FRAME		(FIRST_TIPTILT_MESSAGE+1)

/*
 * Client->Server for setting things to QUAD FRAME mode. No data
 */

#define TIPTILT_QUAD_FRAME		(FIRST_TIPTILT_MESSAGE+2)

/* 
 * Client->Server request for current sub frame. No data.
 */

#define TIPTILT_GET_SUB_FRAME		(FIRST_TIPTILT_MESSAGE+3)

/*
 * Client->Server for setting things to FULL FRAME mode. Data is struct
 * Server Client in answer to a TIPTILT_GET_SUB_FRAME. Data is struct
 */

struct s_tiptilt_sub {
			int x;
			int y;
			int dx;
			int dy;
		     };

#define TIPTILT_SUB_FRAME		(FIRST_TIPTILT_MESSAGE+4)

/* Client->Server select quadrant to use. Data int. */

#define TIPTILT_SET_QUADRANT		(FIRST_TIPTILT_MESSAGE+5)

/* Client->Server request display of a single frame, data */

struct s_tiptilt_display_frame {
			int  pixmult;
			char display_name[256];
			       };

#define TIPTILT_DISPLAY_FRAME		(FIRST_TIPTILT_MESSAGE+6)

/* Client->Server request to clear the X display */

#define TIPTILT_CLEAR_WINDOW		(FIRST_TIPTILT_MESSAGE+7)

/* Client->Server request Surface plot, data name of display */

#define TIPTILT_SURFACE_FRAME		(FIRST_TIPTILT_MESSAGE+8)

/* Client->Server request new bias calculation, data int number of frames. */

#define TIPTILT_MAKE_BIAS		(FIRST_TIPTILT_MESSAGE+9)

/* Client->Server request zeroing of bias. No data. */

#define TIPTILT_ZERO_BIAS		(FIRST_TIPTILT_MESSAGE+10)

/* Client->Server request line plot, data os structure: */

struct s_tiptilt_line{
			int x;
			int y;
			char display_name[256];
		     };

#define TIPTILT_LINE			(FIRST_TIPTILT_MESSAGE+11)

/* Client->Server request movie, data structure */

struct s_movie {
			int type;
			char display[256];
		};

#define FRAME_MOVIE 0
#define SURF_MOVIE  1
#define LINE_MOVIE  2

#define TIPTILT_MOVIE			(FIRST_TIPTILT_MESSAGE+12)

/* Server->Client shows the current status of movie, data in structure */

struct s_movie_status {
			float	min;
			float	max;
			float	frame_rate;
			int	frame_number;
			int	x;
			int	y;
		      };

#define TIPTILT_MOVIE_STATUS		(FIRST_TIPTILT_MESSAGE+13)

/* Server->Clients Stops movie status windows. */

#define TIPTILT_MOVIE_STATUS_STOP     	(FIRST_TIPTILT_MESSAGE+14)

/* CLient->Server for controlling the display socket, data s_socket_name */

#define TIPTILT_OPEN_DISPLAY_SOCKET	(FIRST_TIPTILT_MESSAGE+15)

/*
 * Clinet->Server request number of channels. No data
 * Server->Client Actual number of channels, data integer.
 */

#define TIPTILT_NUMBER_CHANNELS		(FIRST_WOBBLE_MESSAGE+16)

/*
 * Client->Server request chanel names, no data
 */
                                                                                
#define TIPTILT_NAMES  			(FIRST_WOBBLE_MESSAGE+17)
                                                                                
/*
 * Server->Client list of names as per structure
 */
                                                                                
struct s_tiptilt_names {
		char names[NUM_SCOPES][81];
	       };

/*
 * Client->Server set scope names, data structre above.
 */

#define TIPTILT_SCOPE_NAMES		(FIRST_TIPTILT_MESSAGE+18)

/*
 * Client->server request status information on one channel.
 * Data is an integer specifying the chanel.
 */

#define TIPTILT_STATUS			(FIRST_TIPTILT_MESSAGE+19)

/*
 * Server->Client status information, data in following structure
 */

struct s_tiptilt_status {
			int	      det;   /* Which detector */
			unsigned char on;    /* Is the servo on? */
			unsigned char mod_x; /* TRUE or FALSE */
			unsigned char mod_y; /* TRUE or FALSE */
		};

/*
 * Client->server, for turing on/off servo, modulation and signals
 * all have integer (chanel) for arguement
 */

#define TIPTILT_ON			(FIRST_TIPTILT_MESSAGE+20)
#define TIPTILT_OFF			(FIRST_TIPTILT_MESSAGE+21)
#define TIPTILT_MODX			(FIRST_TIPTILT_MESSAGE+22)
#define TIPTILT_MODY			(FIRST_TIPTILT_MESSAGE+23)

/*
 * Client->Server requests to start/stop interrupt callback routine.
 */

#define TIPTILT_START			(FIRST_TIPTILT_MESSAGE+24)
#define TIPTILT_STOP			(FIRST_TIPTILT_MESSAGE+25)

/*
 * Clinet->Server request to reinitialize the camera
 */

#define TIPTILT_INIT			(FIRST_TIPTILT_MESSAGE+26)

/* Client->Server request current faint limit, no data */

#define TIPTILT_GET_FAINT		(FIRST_TIPTILT_MESSAGE+27)

/*
 * CLient->Server Set faint limit
 * Server->Client response to TIPTILT_GET_FAINT, both data float
 */

#define TIPTILT_SET_FAINT               (FIRST_TIPTILT_MESSAGE+28)

/* 
 * Client->Server, request counts seen in system, no data.
 * Server->Client, return counts seen, data of form
 * float counts[NUM_SCOPES];
 */

#define TIPTILT_COUNTS			(FIRST_TIPTILT_MESSAGE+29)

/* Client->Server, requests logging of seeing, no data */

#define TIPTILT_LOG_SEEING		(FIRST_TIPTILT_MESSAGE+30)

/* Client->Server, open or close the log file, no data */

#define TIPTILT_OPEN_LOG		(FIRST_TIPTILT_MESSAGE+31)
#define TIPTILT_CLOSE_LOG		(FIRST_TIPTILT_MESSAGE+32)

/* 
 * Client->Server, request errors seen in system, no data.
 * Server->Client, return errors seen, data of form
 * float errors[NUM_SCOPES];
 */

#define TIPTILT_ERRORS			(FIRST_TIPTILT_MESSAGE+33)

/* Client->Server, set global picture minimum and maximum */

struct s_tiptilt_minmax {
		float min;
		float max;
			};

#define TIPTILT_MINMAX			(FIRST_TIPTILT_MESSAGE+34)

#define LAST_TIPTILT_MESSAGE            (TIPTILT_MINMAX)

/************************************************************************/
/* And now we define the signals that TIPTILT knows about.		*/
/************************************************************************/

#define TIPTILT_SIGNAL_EXIT	1

/************************************************************************/
/* These are not strickly speaking messages, but they are here so       */
/* all clients can see them. These define default ports to use for the  */
/* remote display programs and an incriment to use if things do not     */
/* work.                                                                */
/************************************************************************/

#define IRIMAGE_DISPLAY_PORT_START      3000
#define WOBBLE_DISPLAY_PORT_START       3001
#define S1_DISPLAY_PORT_START           3002
#define S2_DISPLAY_PORT_START           3003
#define E1_DISPLAY_PORT_START           3004
#define E2_DISPLAY_PORT_START           3005
#define W1_DISPLAY_PORT_START           3006
#define W2_DISPLAY_PORT_START           3007
#define IRIMAGE_NIRO_PORT_START      	3008
#define WOBBLE_REF_PORT_START      	3009
#define TIPTILT_DISPLAY_PORT_START     	3010

#define DISPLAY_PORT_INC                  20

#endif
