/********************************************************************************
*   File:       LoadDspFile.c
*   Version:    1.00
*   Author:     Scott Streit
*
*   Abstract:   C library function for load DSP files.
*
*		The LoadDspFile adds the funcionality to the DSPCommand.c 
*		to load files into the boards, pci board, util board,  or 
*		timing board.
*
*		Two C functions,  loadFile() used to load a  file either 
*		to the util board or to the timing board. 
*		And, loadPciFile() to load a file to the pci board.
*
*		Note: that this module may be  used with the DSPCommand.c
*		      module.	
*
*   Revision History:
*       Date            Who   Version    Description
*   --------------------------------------------------------------------------
*		10/31/03		sds		1.7		Initial.
*
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>	//DHB added this
#include "DSPCommand.h"
#include "loadDspFile.h"
/* #include "ErrorString.h" */
#include "../WhichSystem.h"

#ifdef WIN2K
	#include <windows.h>
	#include <winioctl.h>
#endif

/*******************************************************************************
*
*	Function:
*	---------
*		int load_file(int pci_fd, const char *filename, 
*						const char *expected_file_type)
*
* 	Description:	
*	------------
*	This function loads a file into the DSP of the specified controller 
*	board. The controller board can be the  timing board or the utility
*	board as defined by the parameter expected_file_type.
*
*
*       Parameters:
*       -----------
*       pci_fd 			File descriptor for the pci device.
*	filename 		The name of the file to load.
*	expected_file_type 	The controller board to load the file into.
*				May be "timing" or "utility".
* 				This parameter is not case sensitive.
*
*	Returns:
*	--------
*	If successfull returns _NO_ERROR, and if fail returns as _ERROR the
*	source of the fail.
*
*	Version: 	1.00
*	Author:  	Scott Streit
* 	Date:		11/22/1999
*
*   Revision History:
*   -----------------
*       Date            Who   Version    Description
*   --------------------------------------------------------------------------
*       11/22/1999	sds     1.00    Initial
*
*	07/10/2000	jmp		Add support for the error variables
*					reply_value, error_string
*					Take out comments a print outs and
*					resend them to the error variables.
*					Fits the funcionality of the function
*					accordingly the DSPCommand module.
*
*******************************************************************************/
int loadFile(HANDLE pci_fd, char *filename, int validate)
{
	FILE *inFile;
	char line[100];
	char *tokens[20];
	char typeChar;
	char c;
	int tokCnt = 0;
	int boardId = 0;
	int type = 0;
	int addr = 0;
	int data = 0;
	int done = 0;
	int i;
	long pos = 0;
	int status = 0;

	/* First, send the stop command. Otherwise, the controller crashes
	   because it is downloading and executing code while you try to
	   overwrite it. */
	if (doCommand(pci_fd, TIM_ID, STP, DON) == _ERROR) {
		printf("ERROR (LoadDspFile): Cannot stop camera idle!\n");
		return _ERROR;
	}

	/* Set the PCI status bit #1 (X:0 bit 1 = 1). */
	status = doCommand1(pci_fd, PCI_ID, RDM, (X | 0), UNDEFINED);
	if (doCommand2(pci_fd, PCI_ID, WRM, (X | 0), (status | 0x00000002), DON) == _ERROR) {
		printf("ERROR (LoadDspFile): Status set failed, addr: X:0 data: 0x%X\n", (status | 0x00000002));
		return _ERROR;
	}

	/*---------------------------
	  Open the file for reading.
	  ---------------------------*/
	if ((inFile = fopen(filename, "r")) == NULL) 
	{
		printf ("ERROR: Cannot open file: %s \n", filename);
		return _ERROR;
	}

	while (!feof(inFile)) {
		fgets(line, 100, inFile);
		getTokens(line, tokens);
		done = 0;

		if (strcmp(tokens[0], "_DATA") == 0) {
			sscanf(tokens[1], "%c", &typeChar);
			sscanf(tokens[2], "%X", &addr);

			/*Check for a valid start address, which is less than 0x4000 */
			if (addr >= MAX_DSP_START_LOAD_ADDR)
				continue;

			/* Set the DSP memory type */
			if (typeChar == 'X')
				type = X;
			else if (typeChar == 'Y')
				type = Y;
			else if (typeChar == 'P')
				type = P;
			else if (typeChar == 'R')
				type = R;

			while (!done) {
				if ((c = (char)fgetc(inFile)) == '_') {
					done = 1;
					ungetc(c, inFile);
				}
				else {
					ungetc(c, inFile);

					pos = ftell(inFile);

					fgets(line, 100, inFile);
					tokCnt = getTokens(line, tokens);

					for (i=0; i<tokCnt; i++) {
						sscanf(tokens[i], "%X", &data);

						/* Write the data to the controller. */
						if (doCommand2(pci_fd, boardId, WRM, (type | addr), data, DON) == _ERROR) {
							printf("ERROR (LoadDspFile): Write memory failed (0x%X)\n", getError());
							fclose(inFile);
							return _ERROR;
						}

						/* Validate the data if required. */
						if (validate) {
							if (doCommand1(pci_fd, boardId, RDM, (type | addr), data) == _ERROR)
								printf("ERROR (LoadDspFile): Data unsuccessfully written.\n");
						}
						addr++;
					}
				}
			}
		}
		else if (strcmp(tokens[0], "_START") == 0) {
			if (strspn(tokens[1], "TIM") > 0)
				boardId = TIM_ID;
			else if (strspn(tokens[1], "UTIL") > 0)
				boardId = UTIL_ID;
			else {
				printf("ERROR (LoadDspFile): Unexpected file type.\n");
				return _ERROR;
			}
		}
	}

	fclose(inFile);

	/* Clear the PCI status bit #1 (X:0 bit 1 = 0). */
	status = doCommand1(pci_fd, PCI_ID, RDM, (X | 0), UNDEFINED);
	if (doCommand2(pci_fd, PCI_ID, WRM, (X | 0), (status & 0xFFFFFFFD), DON) == _ERROR) {
		printf("ERROR (LoadDspFile): Status clear failed, addr: X:0 data: 0x%X\n", (status | 0x00000002));
		return _ERROR;
	}

	return _NO_ERROR;
}

/************************************************************
  This function takes a line of text and breaks into tokens
  that are separated by spaces.
*************************************************************/
int getTokens(char *textLine, char *tokens[])
{
	int i = 0;

	/* Get the first token */
	tokens[i] = NULL;
	tokens[i] = strtok(textLine, " ");

	/* Get the rest of the tokens */
	while (tokens[i] != NULL) {
		i++;
		tokens[i] = NULL;
		tokens[i] = strtok(NULL, " ");
	}

	/* Returns token count minus one because last value
	   in the file is an empty (wierd) character. */
	return i-1;
}

/*******************************************************************************
*
*	Function:
*	---------
*		int load_pci_file(int pci_fd, const char *filename)
*
* 	Description:	
*	------------
*	This function downloads PCI boot code from a file to the PCI board
* 	DSP program memory space (P). The file download must  be performed
* 	before any  other  controller  actions occur,  otherwise, the  PCI
* 	board's actions are undefined.
*			
*
*       Parameters:
*       -----------
*       pci_fd 		File descriptor for the pci device.
*	filename 	The name of the file to load, e.g. "pci.lod"
*
*	Returns:
*	--------
*	If successfull returns _NO_ERROR, and if fail returns as ERROR the
*	source of the fail.
*
*       Version:	1.00
*       Author:	Scott Streit
* 	Date:		02/01/2000
*
*   Revision History:
*   -----------------
*       Date            Who   Version    Description
*   --------------------------------------------------------------------------
* 	02/01/2000			sds		1.00	Initial
*
* 	02/11/2000			sds				Updated to check that the total number
*										of words to transfer is less than MAX_ 
* 										and that the start addr equals 0.
*
*	07/10/2000			jmp				Add support for the error variables
*										reply_value, error_string. 
*										Change the order of checks. First of 
*										all check if the pci load file can be
*										open, before send any command to the
*										HTF register, otherwise the pci device
*										state can became undefinied.
*										Take out comments a print outs and
*										resend them to the error variables.
*										Fits the funcionality of the function
*										accordingly the DSPCommand module.
*
*	01/03/2002			sds		1.7		Modified to be a win2k dll.
*
******************************************************************************/
int loadPciFile(HANDLE pci_fd, const char *filename)
{
	char input_line[81];
	const char *words_string = " ";
	const char *addr_string = " ";
	const char *fileType = " ";
	const char *tokens[20];
	int data = 0;
	int addr = 0;
	int done_reading_data = 0;
	int reply = 0;
	int word_total = 0;
	int word_count = 0;
	int count = 0;
	int i = 0;
	int failed_count = 0;
	FILE *inFile;
		
	/* Set the PCI board to slave mode. */
#ifdef WIN2K
	ULONG bytesReturned = 0;
	DeviceIoControl(pci_fd, ASTROPCI_GET_HCTR, NULL, 0, &reply, sizeof(reply),
						&bytesReturned, NULL);

	/* Clear the HTF bits and bit 3. */
	reply = reply & HTF_CLEAR_MASK & BIT3_CLEAR_MASK;
		
	/* Set the HTF bits. */
	reply = reply | HTF_MASK;

	DeviceIoControl(pci_fd, ASTROPCI_SET_HCTR, &reply, sizeof(reply), NULL, 0,
						&bytesReturned, NULL);

	/* Inform the DSP that new pci boot code will be downloaded. */
	DeviceIoControl(pci_fd, ASTROPCI_PCI_DOWNLOAD, NULL, 0, NULL, 0, &bytesReturned, NULL);

	/* Set the magic value that says this is a PCI download. */
	reply = 0x00555AAA;
	DeviceIoControl(pci_fd, ASTROPCI_HCVR_DATA, &reply, sizeof(reply), NULL, 0,
						&bytesReturned, NULL);

#else
	ioctl(pci_fd, ASTROPCI_GET_HCTR, &reply);

	/* Clear the HTF bits and bit 3. */
	reply = reply & HTF_CLEAR_MASK & BIT3_CLEAR_MASK;
		
	/* Set the HTF bits. */
	reply = reply | HTF_MASK;

	ioctl(pci_fd, ASTROPCI_SET_HCTR, &reply);

	/* Inform the DSP that new pci boot code will be downloaded. */
	ioctl(pci_fd, ASTROPCI_PCI_DOWNLOAD, 0);

	/* Set the magic value that says this is a PCI download. */
	reply = 0x00555AAA;
	ioctl(pci_fd, ASTROPCI_HCVR_DATA, &reply);
#endif

	/* Open the file for reading. */
	if ((inFile = fopen(filename, "r")) == NULL) {
		sprintf(error_string, "\nError: Cannot open file: %s \n", filename);
		return _ERROR;
	}
		
	while(!done_reading_data) {
		fgets(input_line, 80, inFile);

		if (strstr(input_line, SEARCH_STRING) != NULL) {
			/* Get the next line. */
			fgets(input_line, 80, inFile);

			/* Get the number of words and starting address. */
			words_string = strtok(input_line, " ");
			addr_string = strtok(NULL, " ");

			sscanf(words_string, "%X", &word_total);
			sscanf(addr_string, "%X", &addr);

			/* Check that the number of words is less that 4096 (0x1000). */
			if (word_total > 0x1000) {
				printf("\nError: Number of words to write exceeds DSP memory range.");
				return _ERROR;
			}
			else {
#ifdef WIN2K
				DeviceIoControl(pci_fd, ASTROPCI_HCVR_DATA, &word_total, sizeof(word_total),
									NULL, 0, &bytesReturned, NULL);
#else
				ioctl(pci_fd, ASTROPCI_HCVR_DATA, &word_total);
#endif
			}

			/* Check that the address is equal to 0. */
			if (addr != 0) {
				printf("\nError: Address not equal to zero.");
				exit(1);
			}
			else {
#ifdef WIN2K
				DeviceIoControl(pci_fd, ASTROPCI_HCVR_DATA, &addr, sizeof(addr), NULL, 0,
									&bytesReturned, NULL);
#else
				ioctl(pci_fd, ASTROPCI_HCVR_DATA, &addr);
#endif
				addr = 0;
			}

			/* Throw away the next line (example: _DATA P 000002). */
			fgets(input_line, 80, inFile);
				
			/* Load the data. */
			while (word_count < (word_total - 2)) {
				/* Get the next line, this is the data start. */
				fgets(input_line, 80, inFile);

				/* Check for "_DATA" strings and discard them by     */
				/* reading the next data line, which should be data. */
				if (strstr(input_line, SEARCH_STRING) != NULL)
					fgets(input_line, 80, inFile);
				
				count = 0;
				tokens[count] = strtok(input_line, " ");
					
				if (strstr(tokens[0], "_") != NULL) {
					done_reading_data = 1;
					fseek (inFile, -15, SEEK_CUR);
				}
						
				else {
					while (tokens[count] != NULL) {
						count++;
						tokens[count] = strtok(NULL, " ");
					}
					
					for (i=0; i<count-1; i++) {
						sscanf(tokens[i], "%X", &data);
#ifdef WIN2K
						DeviceIoControl(pci_fd, ASTROPCI_HCVR_DATA, &data, sizeof(data), NULL, 0,
									&bytesReturned, NULL);
#else
						ioctl(pci_fd, ASTROPCI_HCVR_DATA, &data);
#endif
						addr++;
						word_count++;
					}
				}
			}
			done_reading_data = 1;
		}
		
		/* Check the file validity. */
		else if (strspn(input_line, "_START") > 0) {
			fileType = strtok(input_line, " ");
			fileType = strtok(NULL, " ");

			if (strspn(fileType, "PCI") > 0)
				printf("PCI file ... OK.\n");
			else {
				printf("Error: Invalid file type.\n");
				return _ERROR;
			}
		}
	}
	/* Set the PCI board data transfer format (Set HTF bits to 00). */
#ifdef WIN2K
	DeviceIoControl(pci_fd, ASTROPCI_GET_HCTR, NULL, 0, &reply, sizeof(reply),
						&bytesReturned, NULL);
//	reply = (reply & HTF_CLEAR_MASK) | 0x900;
	reply = (reply & 0xCFF) | 0x900;
	DeviceIoControl(pci_fd, ASTROPCI_SET_HCTR, &reply, sizeof(reply), NULL, 0,
						&bytesReturned, NULL);

	fclose(inFile);

	/* Wait for the PCI DSP to finish initialization. */
	DeviceIoControl(pci_fd, ASTROPCI_PCI_DOWNLOAD_WAIT, NULL, 0, &reply, sizeof(reply),
						&bytesReturned, NULL);
#else
	ioctl(pci_fd, ASTROPCI_GET_HCTR, &reply);
	reply = (reply & HTF_CLEAR_MASK) | 0x900;
	ioctl(pci_fd, ASTROPCI_SET_HCTR, &reply);

	fclose(inFile);

	/* Wait for the PCI DSP to finish initialization. */
	ioctl(pci_fd, ASTROPCI_PCI_DOWNLOAD_WAIT, &reply);
#endif

	/* Make sure a DON is received. */
	if (reply != DON)
		sprintf(error_string, "ERROR: Bad reply, expected: DON (0x444F4E), got: 0x%X\n", reply);

	/* Test PCI communications. */
	for (i=0; i<3; i++) {
		doCommand1(pci_fd, PCI_ID, TDL, 0x1234*i, 1234*i);
		failed_count++;
	}

	/* If all the communication tests failed, then report an error. */
	if (failed_count >= 3)
		printf("\nPCI communications test failed.\n");

	return _NO_ERROR;
}
